import numpy as np
import pyvisa
import time


class system():
    inst = ''


def pulsed_sweep_v_dual(start, stop, numpoints, pulsewidth, pulseperiod, limiti, nplc):
    rm = pyvisa.ResourceManager()
    address = "GPIB0::25::INSTR"
    system.inst = rm.open_resource(address)
    system.inst.write("errorqueue.clear()")

    # Konfigurieren Sie die Einstellungen für Kanal A
    system.inst.write("smua.reset()")
    system.inst.write("smua.source.func = smua.OUTPUT_DCVOLTS")

    # if remotesense:
    #     system.inst.write("smua.sense = smua.SENSE_REMOTE")
    # else:
    #     system.inst.write("smua.sense = smua.SENSE_LOCAL")
    system.inst.write("smua.sense = smua.SENSE_LOCAL")

    system.inst.write("smua.source.autorangev = smua.AUTORANGE_OFF")
    stop_v = "smua.source.rangev = {}".format(stop)
    system.inst.write(stop_v)
    system.inst.write("smua.source.levelv = 0")
    system.inst.write("smua.source.limiti = 0.1")

    # Das Deaktivieren von Auto-Ranging und Auto-Zero gewährleistet ein genaues und konsistentes Timing
    system.inst.write("smua.measure.autozero = smua.AUTOZERO_ONCE")
    system.inst.write("smua.measure.autorangei = smua.AUTORANGE_OFF")
    limit_i = "smua.measure.rangei = {}".format(limiti)
    system.inst.write(limit_i)
    nplc_s = "smua.measure.nplc = {}".format(nplc)
    system.inst.write(nplc_s)
    # Ein Timer wird verwendet, um die Messverzögerung einzustellen und
    # die Messung zwischen den beiden SMUs zu synchronisieren, also setzen Sie die eingebaute Verzögerung auf 0.
    system.inst.write("smua.measure.delay = 0")

    # Bereiten Sie die Lesepuffer vor
    system.inst.write("smua.nvbuffer1.clear()")
    system.inst.write("smua.nvbuffer1.collecttimestamps = 1")
    system.inst.write("smua.nvbuffer2.clear()")
    system.inst.write("smua.nvbuffer2.collecttimestamps = 1")

    # Ende der Kanal A Einstellungen

    # Konfigurieren Sie die Einstellungen für Kanal B
    system.inst.write("smub.reset()")
    system.inst.write("smub.source.func = smub.OUTPUT_DCVOLTS")

    # if remotesense:
    #     system.inst.write("smub.sense = smub.SENSE_REMOTE")
    # else:
    #     system.inst.write("smub.sense = smub.SENSE_LOCAL")
    system.inst.write("smub.sense = smub.SENSE_LOCAL")

    system.inst.write("smub.source.autorangev = 0")
    stop_v = "smub.source.rangev = {}".format(stop)
    system.inst.write(stop_v)
    system.inst.write("smub.source.levelv = 0")
    system.inst.write("smub.source.limiti = 0.1")

    # Das Deaktivieren von Auto-Ranging und Auto-Zero gewährleistet ein genaues und konsistentes Timing
    system.inst.write("smub.measure.autozero = smub.AUTOZERO_ONCE")
    system.inst.write("smub.measure.autorangei = 0")
    limit_i = "smub.measure.rangei = {}".format(limiti)
    system.inst.write(limit_i)
    nplc_s = "smub.measure.nplc = {}".format(nplc)
    system.inst.write(nplc_s)
    # Ein Timer wird verwendet, um die Messverzögerung einzustellen und
    # die Messung zwischen den beiden SMUs zu synchronisieren, also setzen Sie die eingebaute Verzögerung auf 0.
    system.inst.write("smub.measure.delay = 0")

    # Bereiten Sie die Lesepuffer vor
    system.inst.write("smub.nvbuffer1.clear()")
    system.inst.write("smub.nvbuffer1.collecttimestamps = 1")
    system.inst.write("smub.nvbuffer2.clear()")
    system.inst.write("smub.nvbuffer2.collecttimestamps = 1")

    # Ende der Kanal A Einstellungen

    # Konfigurieren Sie das Triggermodell
    # durch Drücken der TRIG-Taste auf der Frontplatte wird der Start des Sweeps ausgelöst

    system.inst.write("display.trigger.clear()")

    # Timer 1 controls the pulse period
    num_points = "trigger.timer[1].count = {}".format(numpoints)
    system.inst.write(num_points)
    delay_1 = "trigger.timer[1].delay = {}".format(pulseperiod)
    system.inst.write(delay_1)
    system.inst.write("trigger.timer[1].passthrough = true")
    system.inst.write("trigger.timer[1].stimulus = smua.trigger.SWEEPING_EVENT_ID")

    # Timer 2 kontrolliert die Messung
    system.inst.write("trigger.timer[2].count = 1")
    # Stellen Sie die Messverzögerung lang genug ein, damit die Messungen beginnen, nachdem sich der Impuls beruhigt
    # hat, aber kurz genug, dass sie in die Breite des Impulses passt.
    delay_2 = "trigger.timer[2].delay = {}".format(pulsewidth - (1 / 50) * nplc - 60e-6)
    print(delay_2)
    system.inst.write(delay_2)
    system.inst.write("trigger.timer[2].passthrough = false")
    system.inst.write("trigger.timer[2].stimulus = trigger.timer[1].EVENT_ID")

    # Timer 3 controls the pulse width
    system.inst.write("trigger.timer[3].count = 1")
    delay_3 = "trigger.timer[3].delay = {}".format(pulsewidth)
    system.inst.write(delay_3)
    system.inst.write("trigger.timer[3].passthrough = false")
    system.inst.write("trigger.timer[3].stimulus = trigger.timer[1].EVENT_ID")

    # Konfigurieren Sie das SMUA-Auslösermodell für Sweep
    smua_linearv = "smua.trigger.source.linearv({}, {}, {})".format(start, stop, numpoints)
    system.inst.write(smua_linearv)
    smua_limit_i = "smua.trigger.source.limiti = {}".format(limiti)
    system.inst.write(smua_limit_i)
    system.inst.write("smua.trigger.measure.action = smua.ENABLE")
    system.inst.write("smua.trigger.measure.iv(smua.nvbuffer1, smua.nvbuffer2)")
    system.inst.write("smua.trigger.endpulse.action = smua.SOURCE_IDLE")
    system.inst.write("smua.trigger.endsweep.action = smua.SOURCE_IDLE")
    smua_count = "smua.trigger.count = {}".format(numpoints)
    system.inst.write(smua_count)
    system.inst.write("smua.trigger.arm.stimulus = 0")
    system.inst.write("smua.trigger.source.stimulus = trigger.timer[1].EVENT_ID")
    system.inst.write("smua.trigger.measure.stimulus = trigger.timer[2].EVENT_ID")
    system.inst.write("smua.trigger.endpulse.stimulus = trigger.timer[3].EVENT_ID")
    system.inst.write("smua.trigger.source.action = smua.ENABLE")

    # Konfigurieren Sie das SMUB-Auslösermodell für Sweep
    smub_linearv = "smub.trigger.source.linearv({}, {}, {})".format(start, stop, numpoints)
    system.inst.write(smub_linearv)
    smub_limit_i = "smub.trigger.source.limiti = {}".format(limiti)
    system.inst.write(smub_limit_i)
    system.inst.write("smub.trigger.measure.action = smub.ENABLE")
    system.inst.write("smub.trigger.measure.iv(smub.nvbuffer1, smub.nvbuffer2)")
    system.inst.write("smub.trigger.endpulse.action = smub.SOURCE_IDLE")
    system.inst.write("smub.trigger.endsweep.action = smub.SOURCE_IDLE")
    smub_count = "smub.trigger.count = {}".format(numpoints)
    system.inst.write(smub_count)
    system.inst.write("smub.trigger.arm.stimulus = 0")
    system.inst.write("smub.trigger.source.stimulus = trigger.timer[1].EVENT_ID")
    system.inst.write("smub.trigger.measure.stimulus = trigger.timer[2].EVENT_ID")
    system.inst.write("smub.trigger.endpulse.stimulus = trigger.timer[3].EVENT_ID")
    system.inst.write("smub.trigger.source.action = smub.ENABLE")

    # Ende der Konfiguration Trigger Model

    system.inst.write("smua.source.output = smua.OUTPUT_ON")
    system.inst.write("smub.source.output = smub.OUTPUT_ON")

    # Start the trigger model execution
    start_zeit = time.time()
    system.inst.write("smua.trigger.initiate()")
    system.inst.write("smub.trigger.initiate()")

    # Sweep will not start until the TRIG button is pressed

    # Wait until the sweep has completed
    system.inst.write("waitcomplete()")

    system.inst.write("smua.source.output = smua.OUTPUT_OFF")
    system.inst.write("smub.source.output = smub.OUTPUT_OFF")

    print("Time\tSMUA Voltage\tSMUA Current\tSMUB Voltage\tSMUB Current")
    system.inst.write("printbuffer(1, smua.nvbuffer2.n, smua.nvbuffer2)")
    print(system.inst.read())
    stop_zeit = time.time()
    print(stop_zeit - start_zeit)
    # system.inst.write("printbuffer(1, smua.nvbuffer1.n, smua.nvbuffer1)")
    # print(system.inst.read())
    # system.inst.write("printbuffer(1, smub.nvbuffer2.n, smub.nvbuffer2)")
    # print(system.inst.read())
    # system.inst.write("printbuffer(1, smub.nvbuffer1.n, smub.nvbuffer1)")
    # print(system.inst.read())


if __name__ == '__main__':
    pulsed_sweep_v_dual(0, 10, 101, 1e-3, 5e-3, 0.1, 0.001)
